/*
    Copyright 1998, 1999, 2000, 2001, 2003, 2004 Benhur Stein
    
    This file is part of Paj.

    Paj is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* FilteredEnumerator.m created by benhur on Sat 14-Jun-1997 */

#include "FilteredEnumerator.h"

@implementation FilteredEnumerator

+ (FilteredEnumerator *)enumeratorWithEnumerator:(NSEnumerator *)orgEnum
                                          filter:(NSSet *)names
{
    return [[[self alloc] initWithEnumerator:orgEnum
                                      filter:names] autorelease];
}

+ (FilteredEnumerator *)enumeratorWithEnumerator:(NSEnumerator *)orgEnum
                                          filter:(NSSet *)names
                                        selector:(SEL)sel
{
    return [[[self alloc] initWithEnumerator:orgEnum
                                      filter:names
                                    selector:sel] autorelease];
}

- (id)initWithEnumerator:(NSEnumerator *)orgEnum
                  filter:(NSSet *)names
{
    return [self initWithEnumerator:orgEnum
                             filter:names
                           selector:NULL];
}

- (id)initWithEnumerator:(NSEnumerator *)orgEnum
                  filter:(NSSet *)names
                selector:(SEL)sel
{
    self = [super init];
    if (self) {
        originalEnumerator = [orgEnum retain];
        filter = [names retain];
        selector = sel;
    }
    return self;
}

- (void)dealloc
{
    [originalEnumerator release];
    [filter release];
    [super dealloc];
}

- (id)nextObject
{
    id obj;
    if (selector)
        while ((obj = [originalEnumerator nextObject])
               && (nil != [filter member:[obj performSelector:selector]]))
            ;
    else
        while ((obj = [originalEnumerator nextObject])
               && (nil != [filter member:obj]))
            ;
    return obj;
}
@end
