/*
    Copyright 1998, 1999, 2000, 2001, 2003, 2004 Benhur Stein
    
    This file is part of Paj.

    Paj is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
 */

#include "StatViewer.h"

#include "../General.bproj/Macros.h"

@implementation StatViewer
- initWithController:(PajeTraceController *)c
{
    self = [super initWithController:c];

    if (self != nil) {
        if (![NSBundle loadNibNamed:@"StatViewer" owner:self])
            NSRunAlertPanel(@"StatViewer", @"Couldn't load interface file",
                            nil, nil, nil);
    }
    
    return self;
}

- (void)awakeFromNib
{
    id cell;
    NSSize cellSize = NSZeroSize;
    NSString *cellSizeString;

    [window setFrameAutosaveName:NSStringFromClass([self class])];

    cellSizeString = [[NSUserDefaults standardUserDefaults]
                               stringForKey:@"StatViewerCellSize"];
    if (cellSizeString)
        cellSize = NSSizeFromString(cellSizeString);
    if (!NSEqualSizes(cellSize, NSZeroSize))
        [matrix setCellSize:cellSize];
    
    cell = [[PieCell alloc] initImageCell:nil];
    [cell setRepresentedObject:nil];
    [cell setDataProvider:self];
    [matrix setPrototype:cell];
    [matrix putCell:cell atRow:0 column:0];
    
    // notify that we are a Tool
    [self registerTool:self];
}

- (NSString *)toolName
{
    return @"Stat Viewer";
}

- (void)activateTool:(id)sender
{
    [window makeKeyAndOrderFront:self];
}

- (void)setInputComponent:(PajeComponent *)component
{
    [super setInputComponent:component];
    Assign(filter, component);
}

- (void)hierarchyChanged
{
    NSNotificationCenter *nc;

    [self invalidateValues];

    nc = [NSNotificationCenter defaultCenter];
    [nc removeObserver:self
                  name:@"PajeTimeSelectionChangedNotification"
                object:nil];
    [nc addObserver:self
           selector:@selector(timeSelectionChanged:)
               name:@"PajeTimeSelectionChangedNotification"
             object:[self rootInstance]];
    //NSLog(@"experando notif para %@", [self rootInstance]);
}

- (void)dataChangedForEntityType:(PajeEntityType *)entityType
{
    [self invalidateValues];
}

- (void)setStartTime:(NSDate *)time
{
    Assign(startTime, time);
}

- (void)setEndTime:(NSDate *)time
{
    Assign(endTime, time);
}

- (void)timeSelectionChanged:(NSNotification *)notification
{
#ifdef GNUSTEP
    NSLog(@"StatView recebeu notificacao: %@", notification);
#endif
    [self setStartTime:[[notification userInfo] objectForKey:@"StartTime"]];
    [self setEndTime:[[notification userInfo] objectForKey:@"EndTime"]];
    [durationField setStringValue:[NSString stringWithFormat:@"%.6f s", [endTime timeIntervalSinceDate:startTime]]];
    [self invalidateValues];
}

extern NSDictionary *USER_TYPES;

- (void)invalidateValues
{
    int i=0;
    NSEnumerator *nn;
    id container;
    PieCell *cell;

    if (!filter) return;

    nn = [filter enumeratorOfContainersTyped:[USER_TYPES objectForKey:@"Virtual Node"]
                                 inContainer:[filter rootInstance]];
    while (container = [nn nextObject]) {
        [matrix renewRows:i+1 columns:1];

        cell = [matrix cellAtRow:i column:0];
        [cell setRepresentedObject:container];
        [cell setData:nil];
        [cell setDataProvider:self];
        i++;
    }
    [matrix sizeToCells];
    [matrix setNeedsDisplay];
}

- (void)provideDataForCell:(PieCell *)cell
{
    NSEnumerator *enumerator;
    PajeEntity *entity;
    id array;
    PajeContainer *container;

    if (!filter) return;

    if (!startTime) [self setStartTime:[filter startTime]];
    if (!endTime) [self setEndTime:[filter endTime]];

    container = [cell representedObject];
    NSAssert(container, @"Invalid container in StatViewer cell");

    array = [[StatArray alloc] initWithName:[NSString stringWithFormat:@"%@ %@", [[container entityType] description], [container description]]];

    switch ([[graphSelector selectedItem] tag]) {
        case 1000:
            enumerator = [filter enumeratorOfEntitiesTyped:@"Activity State"
                                               inContainer:container
                                                  fromTime:startTime
                                                    toTime:endTime];
            while (entity = [enumerator nextObject]) {
                StatValue *v;
                NSDate *entityStartTime = [filter startTimeForEntity:entity];
                NSDate *entityEndTime = [filter endTimeForEntity:entity];
                NSDate *firstTime = [startTime laterDate:entityStartTime];
                NSDate *lastTime = [endTime earlierDate:entityEndTime];
                double interval = [lastTime timeIntervalSinceDate:firstTime];
                v = [StatValue valueWithValue:interval
                                        color:[filter colorForEntity:entity]
                                         name:[filter nameForEntity:entity]];
                [array addObject:v];
            }
                break;

        case 0:
            enumerator = [filter enumeratorOfEntitiesTyped:[USER_TYPES objectForKey:@"State"]
                                               inContainer:container
                                                  fromTime:startTime
                                                    toTime:endTime];
            while (entity = [enumerator nextObject]) {
                StatValue *v;
                double duration;
                duration = [[endTime earlierDate:[self endTimeForEntity:entity]] timeIntervalSinceDate:[startTime laterDate:[self startTimeForEntity:entity]]];
                v = [StatValue valueWithValue:duration
                                        color:[filter colorForEntity:entity]
                                         name:[filter nameForEntity:entity]];
                [array addObject:v];
            }
                break;

        case 1:
            enumerator = [filter enumeratorOfEntitiesTyped:[USER_TYPES objectForKey:@"State"]
                                               inContainer:container
                                                  fromTime:startTime
                                                    toTime:endTime];
            while (entity = [enumerator nextObject]) {
                StatValue *v;
                v = [StatValue valueWithValue:1
                                        color:[filter colorForEntity:entity]
                                         name:[filter nameForEntity:entity]];
                [array addObject:v];
            }
                break;

        case 2:
            enumerator = [filter enumeratorOfEntitiesTyped:[USER_TYPES objectForKey:@"Comunications"]
                                               inContainer:[filter rootInstance]
                                                  fromTime:startTime
                                                    toTime:endTime];
            while (entity = [enumerator nextObject]) {
                id size = [entity valueOfFieldNamed:@"Size"];
                if (size) {
                    id source = [filter sourceContainerForEntity:entity];
                    id dest = [filter destContainerForEntity:entity];
                    StatValue *v;
                    if ([source isEqual: container]) {
                        v = [StatValue valueWithValue:[size doubleValue]
                                                color:[filter colorForEntity:entity]
                                                 name:[@"To " stringByAppendingString:[filter nameForEntity:dest]]];
                        [array addObject:v];
                    }
                    if ([dest isEqual: container]) {
                        v = [StatValue valueWithValue:[size doubleValue]
                                                color:[filter colorForEntity:entity]
                                                 name:[@"From " stringByAppendingString:[filter nameForEntity:source]]];
                        [array addObject:v];
                    }
                }
            }
                break;
    }

    [cell setData:array];
    [cell setGraphType:[[graphTypeSelector selectedItem] tag]];
    [cell setShowPercent:[[percentSelector selectedCell] tag]];
    [array release];
}

- (void)changeData:(id)sender
{
    [self invalidateValues];
}

- (void)changeGraph:(id)sender
{
    int rows, cols;
    PieCell *cell;
    int i;

    [matrix getNumberOfRows:&rows columns:&cols];
    for (i=0; i<rows; i++) {
        cell = [matrix cellAtRow:i column:0];
        [cell setGraphType:[[graphTypeSelector selectedItem] tag]];
        [cell setShowPercent:[[percentSelector selectedCell] tag]];
    }
    [matrix setNeedsDisplay];
}

- (void)changeInitialAngle:(id)sender;
{
    [[matrix cells] makeObjectsPerformSelector:@selector(setInitialAngle:)
                                    withObject:sender];
    [matrix display];
}

- (void)windowDidResize:(NSNotification *)notification
{
        NSSize cellSize;
        NSRect matrixFrame;

#if 1
        cellSize = [matrix cellSize];
        matrixFrame = [matrix frame];
        cellSize.width = NSWidth(matrixFrame);
        [matrix setCellSize:cellSize];
        [matrix sizeToCells];
        [matrix setNeedsDisplay];
#else
        [matrix setAutosizesCells:YES];
        [matrix setValidateSize:NO];
#endif
        [matrix setNeedsDisplay];
        cellSize = [matrix cellSize];

        [[NSUserDefaults standardUserDefaults] setObject:NSStringFromSize(cellSize) forKey:@"StatViewerCellSize"];
}

- (void)print:(id)sender
{
//    [[NSPrintOperation printOperationWithView:[window contentView]] runOperation]; 
    [window print:sender];
}
@end
